from context import get_json_value
from util import dumpjs, print_traceback, update_baseconfig_value
from hashlib import sha1
import hmac


def json_response(body, statuscode=200):
    return {
        'status_code': statuscode,
        'body': dumpjs(body),
        'headers': {
            'Content-Type': 'application/json; charset=utf-8'
        }
    }


def error_response(code, realm='user', msg=None):
    resp = {
        'realm': realm,
        'code': 'error.%s.%s' % (realm, code)
    }
    if msg is not None:
        resp['parameters'] = {
            'error': msg
        }
    return json_response(resp, 400)


def api_start_now(easydb_context, logger, parameters):
    try:
        signature = get_json_value(parameters, 'headers.X-Hub-Signature')
        if signature is None:
            logger.debug('X-Hub-Signature missing')
            return error_response('not_authenticated')

        body = get_json_value(parameters, 'body')

        # check crypt algorithm
        crypt = 'sha1='
        if not signature.startswith(crypt):
            logger.debug('X-Hub-Signature invalid')
            return error_response('not_authenticated')

        webhook_hmac = str(easydb_context.get_config(
            'system.auto_keyworder.webhook_hmac'))
        if webhook_hmac is None:
            logger.debug('system.auto_keyworder.webhook_hmac not defined')
            return error_response('not_authenticated', 'system.auto_keyworder.webhook_hmac not defined')

        h = hmac.new(webhook_hmac.encode('utf-8'),
                     body.encode('utf-8'),
                     sha1).hexdigest()
        if crypt+h != signature:
            logger.debug(
                'X-Hub-Signature invalid: does not match %s%s' % (crypt, h))
            return error_response('not_authenticated')

        logger.debug(
            'X-Hub-Signature %s ok: api request authenticated' % signature)

        cursor = easydb_context.get_db_cursor()
        if update_baseconfig_value(cursor,
                                   logger,
                                   _class='system',
                                   _key='auto_keyworder',
                                   _parameter='start_now',
                                   value_type='value_bool',
                                   value=True):
            logger.debug(
                'set baseconfig entry system.auto_keyworder.start_now to True')
        else:
            logger.debug(
                'could not set baseconfig entry system.auto_keyworder.start_now to True')
            return json_response({
                'start_now_set': False
            })

        return json_response({
            'start_now_set': True
        })

    except Exception as e:
        logger.warn('error during api call /start_now: %s' % e.message)
        print_traceback(logger, e)
        return error_response('internal_error', 'api', e.message)
